/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas;

import ideas.domain.Ideas;
import ideas.filters.UnitOfWorkFilter;
import ideas.persistence.AtomicUnitOfWork;
import ideas.persistence.UnitsOfWork;
import ideas.representation.IdeaProvider;
import ideas.resource.IdeaResource;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.jboss.resteasy.plugins.server.servlet.HttpServletDispatcher;

/**
 * @author Keith Webster Johnston [@johnstok]
 *
 */
public class App {

	public static void main(String[] args) throws Exception{
        
			UnitsOfWork unitsOfWork = UnitsOfWork.newInstance();
			AtomicUnitOfWork atomicUnitOfWork = unitsOfWork.newAtomicUnitOfWork();
			    
      unitsOfWork
    	  .decorate(atomicUnitOfWork)
				.createfromListResource( Ideas.singleton() );

		// Configure the RestEasy servlet
        ServletHolder servlet = new ServletHolder(HttpServletDispatcher.class);
        servlet.setInitParameter("resteasy.resources", IdeaResource.class.getName());
        servlet.setInitParameter("resteasy.providers", IdeaProvider.class.getName());
        
        // Bind the servlet to the root context
        ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
        context.setContextPath("/");
        context.addServlet(servlet,"/*");

        context.addFilter(UnitOfWorkFilter.class, "/*", 1);
        
        // Start the server
        Server server = new Server(Integer.valueOf(System.getenv("PORT")));
        server.setHandler(context);
        server.start();
        server.join();   
    }
}