/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.annotations;

/**
 * Defines how expensive it is to instantiate a type or execute a method 
 * 
 * @author Markos Charatzas [@qnoid]
 *
 */
public enum CostValue 
{
	/**
	 * <b>Type:</b> can be discarded and instantiated freely.
	 * <b>Method invocation:</b> has minimum overhead
	 * 
	 */
	INEXPENSIVE, 
	
	/**
	 * <b>Type:</b> consider caching and reuse
	 * <b>Method invocation:</b> consider calling at application startup or
	 * assign a separate thread  
	 */
	EXPENSIVE
}
