/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.domain;

import ideas.persistence.NamedQueryType;
import ideas.persistence.NamedQueryTypes;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;

import org.hibernate.annotations.GenericGenerator;

/**
 * @author Keith Webster Johnston [@johnstok]
 *
 */
@Entity
@NamedQueries({ 
	@NamedQuery(name = "ideas.domain.Idea", query = "from Idea") 
	})
@org.hibernate.annotations.Entity(
		dynamicUpdate = true
)
public class Idea 
{
	/**
	 * Provides all available Idea queries
	 *
	 */
	public static final class NamedQueries
	{
		/**
		 * List all Ideas
		 */
		public static final NamedQueryType<Idea> LIST_ALL = 
				NamedQueryTypes.of("ideas.domain.Idea", Idea.class); 
	}
	
	@Id
	@GeneratedValue(generator="increment")
	@GenericGenerator(name="increment", strategy = "increment")	
	private Long id;
	
	private String title = "";
	private String description = "";
	
	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public String getTitle() {
		return title;
	}
	
	public void setTitle(String title) {
		this.title = title;
	}
	
	public String getDescription() {
		return description;
	}
	
	public void setDescription(String description) {
		this.description = description;
	}
}
