/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.domain;

import ideas.persistence.ListResource;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * A resource of {@link Idea}s
 * 
 * @author Markos Charatzas [@qnoid]
 *
 */
public final class Ideas implements ListResource<Idea>
{
	private static final class IdeasHolder
	{
		private static final Ideas SINGLETON = 
				IdeasHolder.singleton();
		
		public static final Ideas singleton()
		{
			Idea idea = new Idea();
			
			idea.setTitle("Title");
		  idea.setDescription("Description "+new Date());
			
		return Ideas.instance(idea);
		}		
	}

	/**
	 * @return a existing {@link Ideas} with a single Idea
	 */
	public static final Ideas singleton() {
	return IdeasHolder.SINGLETON;
	}
	
	/**
	 * Creates a new {@link Ideas}
	 * 
	 * @param ideas the ideas to be made available
	 * @return a new {@link Ideas} for all the ideas specified
	 */
	public static final Ideas instance(Idea... ideas) {
	return new Ideas(Arrays.asList(ideas));
	}

	private final List<Idea> ideas;
	
	/**
	 * @param ideas
	 */
	public Ideas(List<Idea> ideas) {
		this.ideas = ideas;
	}

	/* (non-Javadoc)
	 * @see ideas.persistence.Resource#get()
	 */
	@Override
	public List<Idea> get() {
	return this.ideas;
	}	
}
