/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.persistence;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;

/**
 * Implementations should provide any operations related to the provided
 * {@link EntityManager} that would like to be contained within a transaction
 * 
 * @param <Result> the resulf of the operation after 
 * {@link #make(EntityManager) make} or {@link Void} if none
 * 
 * @see AtomicOperations
 * @author Markos Charatzas [@qnoid]
 *
 */
public interface AtomicOperation<Result> {

	/**
	 * Makes the operation 
	 * 
	 * @postcondition the transaction associated with the operation will have 
	 * started
	 * 
	 * @param entityManager the entity manager associated with the transaction 
	 * @return the result of the operation as specified by its generic type
	 * @see EntityTransaction#begin()
	 */
	public Result make(EntityManager entityManager);

}
