/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.persistence;

import javax.persistence.Entity;
import javax.persistence.Id;

/**
 * Holds a value for an {@link Id} field that is used to query for an entity 
 *  
 * @param <T> the type annotated with {@link Entity}
 * @see Keys
 * @author Markos Charatzas [@qnoid]
 *
 */
public interface Key<T> 
{
	/**
	 * @return the annotated {@link Entity} type
	 */
	Class<T> type();

	/**
	 * The value of the key must be of the same type as the annotated {@link Id}
	 * field in the specified type {@code <T>}
	 * @return the value of the key
	 */
	Object value();
}
