/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.persistence;

import javax.persistence.NamedQuery;


/**
 * Provides access to {@link NamedQueryTypes}
 * 
 * @author Markos Charatzas [@qnoid]
 *
 */
public final class NamedQueryTypes 
{
	/**
	 * Create a new {@link NamedQueryType}
	 * 
	 * @param name the name of the query
	 * @param type its type
	 * @return a new {@link NamedQueryType}
	 * @see NamedQuery#name()
	 */
	public static <T> NamedQueryType<T> of(final String name, final Class<T> type){
	return new NamedQueryType<T>() {

		@Override
		public String name() {
		return name;
		}

		@Override
		public Class<T> type() {
		return type;
		}
	};
	}
}
