/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.persistence;

/**
 * Use this class to make {@link AtomicOperation}. 
 * 
 * <p>
 * A single instance {@link AtomicUnitOfWork} can be reused to 
 * {@link #make(AtomicOperation) make multiple} {@link AtomicOperation} before
 * marked as {@link #done() done}.
 * 
 * @author Markos Charatzas [@qnoid]
 *
 */
public interface UnitOfWork 
{
	/**
	 * Makes the atomic operation inside a transaction.
	 * 
	 * <p>
	 * The transaction boundaries depend on the implementation
	 * 
	 * @param operation the operation to make
	 * @return the result after making the operation
	 * @throw {@link UnitOfWorkException} if any exception occurs while 
	 * {@link AtomicOperation#make(javax.persistence.EntityManager) make}
	 * @see AtomicOperations 
	 * 
	 */
	public <Result> Result make(AtomicOperation<Result> operation);

	/**
	 * Mark this unit of work as done. Once a unit of work is done, it should 
	 * be discarded.
	 * 
	 * @postcondition calling {@link #make(AtomicOperation)} will throw an 
	 * exception
	 */
	public void done();

}