/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.representation;

import ideas.domain.Idea;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.net.URLDecoder;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ws.rs.Consumes;
import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;

import com.samskivert.mustache.Mustache;
import com.samskivert.mustache.Template;

/**
 * @author Keith Webster Johnston [@johnstok]
 *
 */
@Provider
@Produces({"text/html", "text/html;mode=edit"})
@Consumes("application/x-www-form-urlencoded")
public class IdeaProvider implements MessageBodyWriter<Idea>, MessageBodyReader<Idea> {

	public long getSize(Idea t,
						Class<?> type, 
						Type genericType, 
						Annotation annotations[],
						MediaType mediaType) {
		return -1;
	}

	public boolean isWriteable(Class<?> type, 
							   Type genericType,
							   Annotation annotations[],
							   MediaType mediaType) {
		return Idea.class.equals(type);
	}

	public void writeTo(Idea t, 
					    Class<?> type, 
					    Type genericType, 
					    Annotation annotations[],
			            MediaType mediaType,
			            MultivaluedMap<String, Object> httpHeaders,
			            OutputStream entityStream) throws IOException, WebApplicationException {
		String mode = mediaType.getParameters().get("mode");
		boolean edit = "edit".equals(mode);
		String templateName =
			(edit)
				? "idea-form.mustache"
				: "idea.mustache";
		PrintWriter pw = new PrintWriter(entityStream);
		Template template =
			Mustache.compiler().compile(
					new InputStreamReader(Thread.currentThread().getContextClassLoader().getResourceAsStream(templateName), Charset.forName("UTF-8")));
        template.execute(t, pw); // FIXME: Specify charset.
        pw.close();
	}

	public boolean isReadable(Class<?> type, 
							  Type genericType,
							  Annotation[] annotations,
							  MediaType mediaType) {
		return Idea.class.equals(type);
	}

	public Idea readFrom(Class<Idea> type, 
						 Type genericType,
						 Annotation[] annotations, 
						 MediaType mediaType,
						 MultivaluedMap<String, String> httpHeaders, 
						 InputStream entityStream) throws IOException, WebApplicationException {
		String entity = read(entityStream, mediaType.getParameters().get("charset"));
		Map<String, List<String>> params = parse(entity, "UTF-8"); // FIXME: Determine charset from request.
		Idea i = new Idea();
		i.setTitle(params.get("title").get(0));
		i.setDescription(params.get("description").get(0));
		return i;
	}

	private Map<String, List<String>> parse(String entity, String encoding) throws UnsupportedEncodingException {
		Map<String, List<String>> params = new HashMap<String, List<String>>();
	    String[] pairs = entity.split("\\&");
	    for (int i=0; i<pairs.length; i++) {
	        String[] fields = pairs[i].split("=");
	        String name = URLDecoder.decode(fields[0], encoding);
	        String value = URLDecoder.decode(fields[1], encoding);
	        if (null==params.get(name)) { params.put(name, new ArrayList<String>()); }
	        params.get(name).add(value);
	    }
	    return params;
	}

	private String read(InputStream entityStream, String charset) throws IOException {
		final ByteArrayOutputStream baos = new ByteArrayOutputStream();
        copy(entityStream, baos);
        final String entity =
            new String(
                baos.toByteArray(),
                (null==charset) ? "UTF8" : charset).trim();
		return entity;
	}

	public static void copy(final InputStream is, 
							final OutputStream os) throws IOException {
		final byte[] buffer = new byte[8*1024];
		while (true) {
			final int amountRead = is.read(buffer);
			if (amountRead == -1) { break; }
			os.write(buffer, 0, amountRead);
		}
	}
}
