/* 
 * This file is part of ideas.
 *
 *  ideas is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  ideas is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with ideas.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * (c) Keith Webster Johnston & Markos Charatzas 
 */
package ideas.resource;

import ideas.annotations.UnitOfWorkType;
import ideas.domain.Idea;
import ideas.persistence.Keys;
import ideas.persistence.LongUnitOfWork;
import ideas.persistence.UnitsOfWork;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriBuilder;

/**
 * @author Keith Webster Johnston [@johnstok]
 *
 */
@Path("/idea")
@Produces("text/html")
public class IdeaResource 
{
	@UnitOfWorkType
	private static LongUnitOfWork unitOfWork;
	
	private final UnitsOfWork unitOfWorks = UnitsOfWork.newInstance(); 
	
	   @GET
	   public Idea get() {
	  	 return this.unitOfWorks.decorate(unitOfWork).read(Keys.of(Idea.class, 1l));
	   }

		@POST
	   @Consumes("application/x-www-form-urlencoded")
	   public Response update(Idea update) {

			Idea idea = this.unitOfWorks.decorate(unitOfWork).read(Keys.of(Idea.class, 1l));
			
			idea.setTitle( update.getTitle() );
			idea.setDescription( update.getDescription() );
			
		   return Response.seeOther(UriBuilder.fromResource(IdeaResource.class).build()).build();
	   }
	   
	   @GET
	   @Path("/edit")
	   @Produces("text/html;mode=edit")
	   public Idea getEditable() {
	      return this.unitOfWorks.decorate(unitOfWork).read(Keys.of(Idea.class, 1l));
	   }
}
