/* (c) Keith Webster Johnston & Markos Charatzas 
 * All Rights Reserved.
 */
package ideas.persistence;

import ideas.domain.Idea;

import java.util.List;

import javax.persistence.EntityManager;

import junit.framework.Assert;

import org.junit.Ignore;
import org.junit.Test;

/**
 * @author Markos Charatzas [@qnoid]
 *
 */
public class UnitOfWorkTest 
{
  private static final UnitsOfWork UNIT_OF_WORKS = UnitsOfWork.newInstance();


  private void givenUnitOfWorkAtomicOperationExceptionThrowUnitOfWorkException(UnitOfWork unitOfWork)
  {
    AtomicOperation<Void> operation = new AtomicOperation<Void>()
        {      
          @Override
          public Void make(EntityManager entityManager)
          {
            throw new RuntimeException();
          }
        };
        
    unitOfWork.make(operation);
  }
  
  @Test
  public void testLongUnitOfWork() 
  {   
    UnitOfWorkDecorator unitOfWork = 
        UNIT_OF_WORKS.decorate( UNIT_OF_WORKS.newLongUnitOfWorkBuilder().build() );

    Idea idea = unitOfWork.read(Keys.of(Idea.class, 1l));
    
    Idea update = new Idea();
    update.setTitle("foo");
    update.setDescription("bar");
    
    idea.setTitle( update.getTitle() );
    idea.setDescription( update.getDescription() );
    
    Assert.assertEquals("foo", unitOfWork.read(Keys.of(Idea.class, 1l)).getTitle());
    Assert.assertEquals("bar", unitOfWork.read(Keys.of(Idea.class, 1l)).getDescription());
    
    unitOfWork.done();
  }
  
  /*
   * Need to setup mock as to mock EntityManager
   */
  @Ignore
	public void testGivenLongUnitOfWorkAtomicOperationExceptionAssertUnitOfWorkException() 
	{
    UnitsOfWork unitOfWorks = UnitsOfWork.newInstance();

		UnitOfWork newLongUnitOfWork = unitOfWorks.newLongAtomicUnitOfWork(null);

		try{
		  givenUnitOfWorkAtomicOperationExceptionThrowUnitOfWorkException(newLongUnitOfWork);
		}
		catch (UnitOfWorkException e) {
      Assert.assertNotNull(e);
    }
		finally{
		  newLongUnitOfWork.done();
		}
	}

	/*
	 * Need to setup mock as to mock EntityTransaction
	 */
  @Ignore
	public void testGivenAtomicUnitOfWorkAtomicOperationExceptionAssertUnitOfWorkException() 
	{
	  AtomicUnitOfWork newAtomicUnitOfWork = new AtomicUnitOfWork(null, null);
	  
    try{
      givenUnitOfWorkAtomicOperationExceptionThrowUnitOfWorkException(newAtomicUnitOfWork);
    }
    catch (UnitOfWorkException e) {
      Assert.assertNotNull(e);
    }
    finally{
      newAtomicUnitOfWork.done();
    }
	  
	}

  @Test
  public void testGivenPersistedAssertList() throws Exception 
  {
    UnitsOfWork unitsOfWork = UnitsOfWork.newInstance();
    
    UnitOfWorkDecorator longUnitOfWork = 
          unitsOfWork.decorate(unitsOfWork.newLongUnitOfWork());
    
    longUnitOfWork.createFromVarrargs( new Idea(), new Idea() );
    
    List<Idea> result = longUnitOfWork.list( Idea.NamedQueries.LIST_ALL );
            
    Assert.assertEquals(2, result.size());
  } 
}
